# Copyright (C) 2022 - 2024 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
"""PyHPS JMS resource subpackage."""
from .algorithm import Algorithm
from .file import File
from .fitness_definition import FitnessDefinition, FitnessTermDefinition
from .job import Job
from .job_definition import JobDefinition
from .license_context import LicenseContext
from .operation import Operation
from .parameter_definition import (
    BoolParameterDefinition,
    FloatParameterDefinition,
    IntParameterDefinition,
    ParameterDefinition,
    StringParameterDefinition,
)
from .parameter_mapping import ParameterMapping
from .permission import Permission
from .project import Project
from .selection import JobSelection
from .task import Task
from .task_definition import (
    HpcResources,
    Licensing,
    ResourceRequirements,
    Software,
    SuccessCriteria,
    TaskDefinition,
)
from .task_definition_template import (
    TaskDefinitionTemplate,
    TemplateInputFile,
    TemplateOutputFile,
    TemplateProperty,
    TemplateResourceRequirements,
)
